package src

import (
	"bufio"
	"encoding/json"
	"fmt"
	"log"
	"os"
	"path/filepath"
	"strconv"
	"strings"
	"time"
)

func OnCommand() {
	command := os.Args[1]
	switch command {
	case "help":
		showHelp()
	case "list":
		showList()
	case "add":
		addConf()
	case "del":
		delConf()
	case "edit":
		editConf()
	case "get":
		onGet()
	case "-s":
		onServerCommand()
	default:
		log.Fatalf("不支持的指令: %s\n您可以使用 help 查看帮助", command)
	}
}

// 帮助
func showHelp() {
	fmt.Printf("help\t查看帮助\n")
	fmt.Printf("list\t查看配置列表\n")
	fmt.Printf("add\t添加配置\n")
	fmt.Printf("del\t删除配置\n")
	fmt.Printf("edit\t修改配置\n")
	fmt.Printf("get\t从获取端获取证书,会自动判断有效期\n")
	fmt.Printf("\t[-f]\t从获取端获取证书,不判断有效期\n")
	fmt.Printf("get all\t从获取端获取证书,会自动判断有效期\n")
	fmt.Printf("\t[-f]\t从获取端获取证书,不判断有效期\n")
	fmt.Printf("-s list\t查看服务端已配置的Domain名称\n")
}

// 显示配置列表
func showList() {
	domains := GetClientConfig().Domains
	if len(domains) == 0 {
		fmt.Println("暂无配置; 您可以使用 add 命令添加配置")
		return
	}
	for i := range domains {
		domain := domains[i]
		fmt.Printf("%d. %s\n", i, domain.Name)
		fmt.Printf("  - 证书文件: %s\n", domain.CertFile)
		fmt.Printf("  - 密钥文件: %s\n", domain.KeyFile)
		fmt.Printf("  - 详情文件: %s\n", domain.InfoFile)
		fmt.Println()
	}
}

// 添加配置
func addConf() {
	domain := &Domain{}
	name := scanConf("请输入服务端已配置名称;可以通过 -s list 查看", "名称不能为空")
	domain.Name = name
	domain.CertFile = scanConf("请输入证书文件存放全路径;如: /data/cert/fullchain.pem", "证书文件路径不能为空")
	domain.KeyFile = scanConf("请输入私钥文件存放全路径;如: /data/cert/privkey.pem", "私钥文件路径不能为空")
	domain.InfoFile = scanConf("请输入详情文件存放全路径;如: /data/cert/info.json", "详情文件路径不能为空")
	config := GetClientConfig()
	config.Domains = append(config.Domains, *domain)
	WriteConfig()
	fmt.Println("添加成功")
	showList()
}

// 删除配置
func delConf() {
	fmt.Println("当前配置: ")
	showList()
	indexInt, _ := selectDomain("请输入要删除的序号")
	config := GetClientConfig()
	config.Domains = append(config.Domains[:indexInt], config.Domains[indexInt+1:]...)
	WriteConfig()
	fmt.Println("删除成功")
	showList()
}

// 修改配置
func editConf() {
	fmt.Println("当前配置")
	showList()
	_, domain := selectDomain("请输入要修改的序号")
	msg := fmt.Sprintf("请输入服务端已配置名称;可以通过 -s list 查看\n当前配置: %s\n", domain.Name)
	domain.Name = scanConf(msg, "名称不能为空")
	certFile := fmt.Sprintf("请输入证书文件存放全路径;如: /data/cert/fullchain.pem\n当前配置: %s\n", domain.CertFile)
	domain.CertFile = scanConf(certFile, "证书文件路径不能为空")
	msg = fmt.Sprintf("请输入私钥文件存放全路径;如: /data/cert/privkey.pem\n当前配置: %s\n", domain.KeyFile)
	domain.KeyFile = scanConf(msg, "私钥文件路径不能为空")
	msg = fmt.Sprintf("请输入详情文件存放全路径;如: /data/cert/info.json\n当前配置: %s\n", domain.InfoFile)
	domain.InfoFile = scanConf(msg, "详情文件路径不能为空")
	WriteConfig()
	fmt.Println("修改成功")
	showList()
}

func onGet() {
	fmt.Println("当前配置")
	showList()
	args := os.Args
	if len(args) < 3 {
		_, domain := selectDomain("请选择要获取证书的配置编号")
		getCert(domain, false)
		return
	}
	if args[2] == "all" {
		getCertAll()
		return
	}
	if args[2] == "-f" {
		_, domain := selectDomain("请选择要获取证书的配置编号")
		getCert(domain, true)
		return
	}

	fmt.Printf("参数错误, get指令不支持 %s 参数\n", args[3])
}

func getCertAll() {
	args := os.Args
	isForce := false
	if len(args) < 4 {
		isForce = false
	} else {
		if args[3] == "-f" {
			isForce = true
		}
	}
	msg := "获取所有已配置的证书"
	if isForce {
		msg = "强制" + msg
	}
	log.Println(msg)
	for _, domain := range GetClientConfig().Domains {
		getCert(&domain, isForce)
	}
}

func getCert(domain *Domain, isForceGet bool) {
	msg := "开始获取 %s 的证书信息."
	if isForceGet {
		msg = msg + " 不校验本地证书信息,直接获取."
	}
	log.Printf(msg, domain.Name)
	isDoGet := isForceGet

	certFile := domain.CertFile
	keyFile := domain.KeyFile
	infoFile := domain.InfoFile

	isDoGet = isDoGet || !isExist(certFile)
	isDoGet = isDoGet || !isExist(keyFile)
	isDoGet = isDoGet || !isExist(infoFile)

	if !isDoGet {
		info, _ := readInfoFile(infoFile)
		if info == nil {
			isDoGet = true
		}
		if !isDoGet {
			if info.Info.NotAfter.Sub(time.Now()) < 7*24*time.Hour {
				isDoGet = true
			}
		}
	}
	if !isDoGet {
		fmt.Println("日志文件已存在, 无需更新.\n如果需要强制更新, 请使用命令: get -f")
		return
	}
	domainData, err := doGetCert(domain.Name)
	if err != nil {
		fmt.Println("从服务端获取配置失败", err)
		return
	}
	writeCert(domain, domainData)
	fmt.Printf("证书相关文件写出成功,配置名称: %s\n", domain.Name)
}

func doGetCert(name string) (*DomainData, error) {
	config := GetClientConfig()
	server := config.Server
	token, encryptToken := GenToken()
	url := server + "/api/v1/cert?name=" + name + "&token=" + encryptToken
	data, err := httpGet(url, token)
	if err != nil {
		return nil, err
	}
	result := DomainData{}
	err = json.Unmarshal([]byte(data), &result)
	if err != nil {
		return nil, err
	}
	return &result, nil
}

// 读取用户输入
func scanConf(msg string, errMsg string) string {
	for {
		log.Println(msg)
		reader := bufio.NewReader(os.Stdin)
		name, err := reader.ReadString('\n')
		if err != nil {
			fmt.Println("读取失败;", err)
			continue
		}
		name = strings.Trim(name, "\r\n")
		if name == "" {
			fmt.Println(errMsg)
			continue
		}
		return name
	}
}

// 选择域名
func selectDomain(msg string) (int, *Domain) {
	index := scanConf(msg, "序号不能为空")
	indexInt, err := strconv.Atoi(index)
	if err != nil {
		log.Fatal("序号错误")
	}
	config := GetClientConfig()
	if indexInt >= len(config.Domains) || indexInt < 0 {
		log.Fatal("序号超出范围")
	}
	return indexInt, &config.Domains[indexInt]
}

// 判断文件是否存在
func isExist(file string) bool {
	_, err := os.Stat(file)
	return !os.IsNotExist(err)
}

// 读取证书详情文件
func readInfoFile(file string) (*CertInfo, error) {
	readFile, err := os.ReadFile(file)
	if err != nil {
		return nil, err
	}
	var certInfo *CertInfo
	err = json.Unmarshal(readFile, &certInfo)
	return certInfo, err
}

// 写出证书
func writeCert(domain *Domain, data *DomainData) {
	certFile := domain.CertFile
	keyFile := domain.KeyFile
	infoFile := domain.InfoFile

	mkFileDir(certFile)
	mkFileDir(keyFile)
	mkFileDir(infoFile)

	fullchain := data.Fullchain
	key := data.Key
	info := data.Info

	if err := os.WriteFile(certFile, []byte(fullchain), 0755); err != nil {
		log.Fatal("写出证书文件失败!", err)
	}
	if err := os.WriteFile(keyFile, []byte(key), 0755); err != nil {
		log.Fatal("写出秘钥文件失败!", err)
	}
	if err := os.WriteFile(infoFile, []byte(info), 0755); err != nil {
		log.Fatal("写出证书详情文件失败!", err)
	}
}

// 创建文件所在的目录
func mkFileDir(file string) {
	dir := filepath.Dir(file)
	if _, err := os.Stat(dir); os.IsNotExist(err) {
		fmt.Printf("证书文件夹 %s 不存在，自动创建...\n", dir)
		err := os.MkdirAll(dir, 0755)
		if err != nil {
			log.Fatal("创建证书文件夹失败!", err)
		}
	}
}
